#!/bin/bash
# OSSEC Agent installer
# Name: ossec-installer.sh
# Copyright Atomicorp 2025
# License: Commercial. Unauthorized redistribution prohibited.

###################
# Global variables
###################

ROOT_UID=0
USE_AWS=0
VERSION=8.3.17
AWS_IP="169.254.169.254"
O_CONF="/var/ossec/etc/ossec.conf"
LOG=/var/log/atomicorp.log
HTTP="http"
HTTP_ENROLL="false"



###################
# Functions
###################

function show_help() {

	echo
	echo "Atomicorp OSSEC Agent installer ($VERSION)"
	echo "Usage: $0 <server_ip>"
	echo "  example: $0 10.11.12.13"
	echo "  or"
	echo "  example: $0 --server 10.11.12.13"

	echo
	echo
	echo "options (deprecated):"
	echo "	<server_ip>	          IP of server hub"
	echo "	<auth_ip>	optional. IP of registration service. Default: <server_ip>"
	echo "	<protocol>	optional. Protocol tcp or udp"
	echo "	<port>		optional. Port for remoted"
	echo "	<group>		optional. Group to assign agent"
	echo
	echo "command line parameters:"
	echo "	--server <server_ip>       		IP/Hostname of the server hub"
	echo "	--auth <auth_ip>           		IP/Hostname of the registration service. Default: <server_ip>"
	echo "	--password <password>      		Registration authentication password"
	echo "	--port <port>              		Port for remoted"
	echo "	--protocol <tcp|udp>       		Protocol for remoted"
	echo
	echo "	--agent-name <name>        		Set agent name [Default: hostname]"
	echo "	--enable-unattended-upgrades		Enable unattended upgrade support (Debian/Ubuntu only)"
	echo "	--exclude-clamav			Exclude ClamAV packages (RPM only)"
	echo "	--default-name             		Prepend the external IP of the agent to agent name"
	echo "	--group  <groupname>       		Group to assign agent"
	echo "	--https                    		Use https for installation"
	echo "	--install-only             		Only install the agent, do not register"
	echo "	--opencsw-repo <local|official>      		Use local opencsw mirror, or official repo (Solaris only)"
	echo "	--proxy <disable|url>      		Proxy settings. Proxy URL, or disable proxy for this installation"
	echo "	--use-aws                  		Use the AWS instance ID for the agent name"
	echo "	--version                  		Show Version"
	echo "	--https-enrollment               	Register agent over HTTPS"
	echo
	echo "environment variables"
	echo "	example: export SERVER_IP=10.11.12.13 $0"
	echo
	echo "	AUTH_IP  	IP of server registration service. "
	echo "	GROUP	  	Group to assign agent"
	echo "	PASSWORD  	Registration authentication password"
	echo "	PROTOCOL  	Protocol tcp or udp"
	echo "	PORT	  	Port for remoted"
	echo "	SERVER_IP  	IP of server hub"
	echo

}

# log and messaging facility
# If type is set to log, and DEBUG is true, then the message will be echoed to the console
# if type is log, only write to the log file
# if type is debug, only write to the console
show_msg() {
	msg=$1

	if [[ $2 == "log" ]]; then
		if [[ $DEBUG ]]; then
			echo "$(date) DEBUG: $msg"
		fi
		echo "$(date) $msg" >> $LOG
	elif [[ $2 == "error" ]]; then
		echo $msg
		echo "$(date) $msg" >> $LOG
	elif [[ $2 == "debug" ]]; then
		echo "$(date) DEBUG: $msg"
		echo "$(date) DEBUG: $msg" >> $LOG
	else 
		echo $msg
	fi
}

function detect_downloader() {
    if [ -f /bin/curl ]; then
        GET="/bin/curl -s -O"
        DOWNLOADER="/bin/curl"
    elif [ -f /usr/bin/curl ]; then
        GET="/usr/bin/curl -s -O"
        DOWNLOADER="/usr/bin/curl"
    elif [ -f /usr/local/bin/curl ]; then
        GET="/usr/local/bin/curl -s -O"
		DOWNLOADER="/usr/local/bin/curl"
    elif [ -f /bin/wget ]; then
        GET="/bin/wget -q"
		DOWNLOADER="/bin/wget"
    elif [ -f /usr/bin/wget ]; then
        GET="/usr/bin/wget -q"
		DOWNLOADER="/usr/bin/wget"
    elif [ -f /opt/freeware/bin/curl ]; then
        GET="/opt/freeware/bin/curl -s -O"
		DOWNLOADER="/opt/freeware/bin/curl"
	elif [ -f /opt/freeware/bin/wget ]; then
		GET="/opt/freeware/bin/wget -q"
		DOWNLOADER="/opt/freeware/bin/wget"
	elif [ -f /usr/sfw/bin/wget ]; then
		GET="/usr/sfw/bin/wget -q"
		DOWNLOADER="/usr/sfw/bin/wget"
    elif [ -f /opt/freeware/bin/wget ]; then
        GET="/opt/freeware/bin/wget -q"
		DOWNLOADER="/opt/freeware/bin/wget"
    elif [ -f /usr/sfw/bin/wget ]; then
        GET="/usr/sfw/bin/wget -q"
		DOWNLOADER="/usr/sfw/bin/wget"
    else
		show_msg "ERROR: downloader not detected. Exiting" error
        exit 1
    fi
	show_msg "GET=$GET" log
}

function request_agent_key() {

	server_ip="$1"
	show_msg "Requesting agent key from $server_ip using request_agent_key" log


	if [ -n "$AGENT_NAME" ]; then
	    local url="https://${server_ip}:${PORT}/agent_register/?key=none&hostname=${agent_name}"
	else
	    local url="https://${server_ip}:${PORT}/agent_register/?key=none&hostname=$(hostname)"
	fi

	export PATH=/opt/csw/bin:/opt/freeware/bin:$PATH
    	cmd="curl -k -o /var/ossec/etc/client.keys.tmp \"$url\""

	eval "$cmd"
	local status=$?


	if [[ $status -eq 0 ]]; then
       		echo "Request successful. Response saved to $output_file."
		awk -F\' '{print $2}' /var/ossec/etc/client.keys.tmp > /var/ossec/etc/client.keys
        	return 0
	else
        	echo "ERROR: Request failed with status code $status."
        	return 1
	fi
}


# Smoketests
function smoketest() {
    # Log and attempt to download diag.sh
    show_msg "Downloading diag.sh for smoketest..." log

    if [[ "$DOWNLOADER" == *curl* ]]; then
        $DOWNLOADER -f -o diag.sh ${HTTP}://${SERVER_IP}/tests/diag.sh 2>>$LOG
    elif [[ "$DOWNLOADER" == *wget* ]]; then
        $DOWNLOADER -O diag.sh ${HTTP}://${SERVER_IP}/tests/diag.sh 2>>$LOG
    else
        show_msg "ERROR: Unsupported downloader: $DOWNLOADER" error
        exit 1
    fi

    # Check if the download command was successful
    if [[ $? -ne 0 ]]; then
		echo
        show_msg "ERROR: Failed to download diag.sh($DOWNLOADER ${HTTP}://${SERVER_IP}/tests/diag.sh)" error
		echo -n "  "
		show_msg "NOTE: If this is a new installation of the hub server it may still be initializing its repositories" error
		echo
        exit 1
    fi

    # Ensure the file exists and is not empty
    if [[ ! -f diag.sh || ! -s diag.sh ]]; then
		echo
        show_msg "ERROR: The downloaded diag.sh file is missing or empty." error
		echo
        exit 1
    fi

    # Move and set executable permissions
    mv diag.sh /var/ossec/bin/
    if [[ $? -ne 0 ]]; then
		echo
        show_msg "ERROR: Failed to move diag.sh to /var/ossec/bin/" error
		echo
        exit 1
    fi

    chmod +x /var/ossec/bin/diag.sh
    if [[ $? -ne 0 ]]; then
		echo
        show_msg "ERROR: Failed to set executable permissions for /var/ossec/bin/diag.sh" error
		echo
        exit 1
    fi

    # Run the diagnostic script and log the output
    show_msg "Running diag.sh..." log
    /var/ossec/bin/diag.sh agent | tee -a $LOG

    # Check if diag.sh ran successfully
    if [[ $? -ne 0 ]]; then
        show_msg "ERROR: diag.sh execution failed." error
        exit 1
    fi

    show_msg "Smoketest completed successfully." log
}


function password_auth() {
	if [ -n "$PASSWORD" ]; then
		echo "$PASSWORD" > /var/ossec/etc/authd.pass
		chmod 640 /var/ossec/etc/authd.pass
		chown root.ossec /var/ossec/etc/authd.pass
		show_msg "Password added to /var/ossec/etc/authd.pass" log
	fi
}

# Hub check
function hub_check() {
    # List of packages to check
    packages=("awp" "awp-hub-utils" "ossec-hids-server")
    
    for pkg in "${packages[@]}"; do
        if rpm -q $pkg &> /dev/null; then
            show_msg "ERROR: Package $pkg is installed. Are you attempting agent installation on a hub? Exiting."
            exit 1
        fi
    done

    show_msg "Hub packages not detected. Proceeding with agent installation."
}

app_error() {
	RETVAL=$1
	MSG=$2
	if [ $RETVAL -ne 0 ];then
		echo
		show_msg "  ERROR: $MSG ($RETVAL)" error
		echo
		exit 1
   	fi
}

app_error_authd() {
	RETVAL=$1
	MSG=$2
	if [ $RETVAL -ne 0 ];then
		echo
		show_msg "  ERROR: $MSG ($RETVAL)" error
		echo
		show_msg "###############################################" error
		show_msg "Running Diagnostics: " error
		show_msg "###############################################" error
		show_msg
		show_msg "  Testing agent-auth validity: " error
		/var/ossec/bin/agent-auth -h | tee -a $LOG
		if [ $? -eq 0 ]; then
			echo
			show_msg "  Testing agent-auth connectivity: " error
			/var/ossec/bin/agent-auth -m ${AUTH_IP} | tee -a $LOG
			echo
		fi
		echo

		if [ -f /usr/sbin/getenforce ]; then
			echo -n "  Getting SELinux state: "  | tee -a $LOG
			getenforce | tee -a $LOG
		fi
		if [ -f /usr/bin/audit2allow ] ; then
			echo
			echo -n "  Running audit2allow please be patient: "	| tee -a $LOG
			audit2allow -a > /root/audit2allow-ossec.log
			echo "done"| tee -a $LOG
			echo
			cat /root/audit2allow-ossec.log| tee -a $LOG
		fi
		echo

		exit 1
		
	fi
}

aws_register () {
  echo ""
  echo "inside aws_register function."
  echo
  #this is held over from the previous version. IF this command works then it finishes method one. OTherwise, attempt method 2.
  echo "Attempting to determine instance id using IMDSv1 method..."
  curl -fs ${HTTP}://$AWS_IP/latest/meta-data/instance-id
  ret=$?
  if [ "$ret" -ne 0 ]; then
		echo
		#method 2 needs a token to my knowledge, so it gets one and does this curl.
		show_msg " ERROR: Could not determine instance-id using IMDSv1 method. Attempting IMDSv2 method..." error
		TOKEN=curl -fs http://169.254.169.254/latest/api/token -H "X-aws-ec2-metadata-token-ttl-seconds: 21600"
		instance_id=$(curl -fs http://169.254.169.254/latest/meta-data/instance-id -H "X-aws-ec2-metadata-token: $TOKEN")
		ret=$?
		#if method 2 fails, just exit.
		if [ "$ret" -ne 0 ]; then 
			echo 
			show_msg " ERROR: Could not determine instance-id using IMDSv2 method. Exiting..." error
			exit 1
		else 
			#method 2 success block, similar to method 1.
			echo
			echo "Success, authenticating..."
			echo
			#register agent
			if [ "$HTTP_ENROLL" = true ]; then

				/var/ossec/bin/agent-auth -m ${AUTH_IP} -A ${instance_id} -G ${GROUP} -H | tee -a $LOG
				app_error_authd ${PIPESTATUS[0]} "  aws_register: agent authorization failed"
			else
				# Commands if HTTP_ENROLL is not true
				/var/ossec/bin/agent-auth -m ${AUTH_IP} -A ${instance_id} -G ${GROUP} | tee -a $LOG
      			app_error_authd ${PIPESTATUS[0]} "  aws_register: agent authorization failed"
			fi
  	   		show_msg "AGENT_NAME=$instance_id" log
		fi
  else 
		#method 1 success block
		echo
		echo "Success, authenticating..."
		echo
		#Obtain Agent Name
		AGENT_NAME=$(curl -fs ${HTTP}://$AWS_IP/latest/meta-data/instance-id)

		#Register the Agent
		if [ "$HTTP_ENROLL" = true ]; then
				# Commands to execute if HTTP_ENROLL is true
				/var/ossec/bin/agent-auth -m ${AUTH_IP} -A ${AGENT_NAME} -G ${GROUP} -H | tee -a $LOG
       			app_error_authd ${PIPESTATUS[0]} "  aws_register: agent authorization failed"
		else
			# Commands if HTTP_ENROLL is not true
			/var/ossec/bin/agent-auth -m ${AUTH_IP} -A ${AGENT_NAME} -G ${GROUP} | tee -a $LOG
			app_error_authd ${PIPESTATUS[0]} "  aws_register: agent authorization failed"
		fi
  	   show_msg "AGENT_NAME=$AGENT_NAME" log
  fi
}


ossec_conf_update(){
	MSG="configuration failed for Hub server destination"
	# Test for ossec.conf server-ip
	if ! grep  ${SERVER_IP} $O_CONF >/dev/null; then
	    if grep address $O_CONF >/dev/null; then
		    $SED -i "s/<address>.*/<address>${SERVER_IP}<\/address>\n      <port>${PORT}<\/port>/g" $O_CONF
		    app_error ${PIPESTATUS[0]} $MSG
		    $SED -i "s/<protocol>.*/<protocol>${PROTOCOL}<\/protocol>/g" $O_CONF
		    app_error ${PIPESTATUS[0]} $MSG
		    $SED -i "s/<manager_address>.*/<manager_address>${AUTH_IP}<\/manager_address>/g" $O_CONF
		    app_error ${PIPESTATUS[0]} $MSG
	    # Legacy
	    elif grep server-ip $O_CONF >/dev/null; then
		    $SED -i "s/<server-ip>.*/<server><address>${SERVER_IP}<\/address><port>${PORT}<\/port><protocol>${PROTOCOL}<\/protocol><\/server>/g" $O_CONF
		    app_error ${PIPESTATUS[0]} $MSG
	    fi
	fi

	# if the AGENT_NAME is set, then update the agent_name value in the config or add it if it doesn't exist
	if [ -n "$AGENT_NAME" ]; then
		if grep "<agent_name>" $O_CONF >/dev/null; then
			$SED -i "s/<agent_name>.*/<agent_name>${AGENT_NAME}<\/agent_name>/g" $O_CONF
			app_error ${PIPESTATUS[0]} "  ossec_conf_update: agent_name update failed"
		else
			# Look for the <enrollment> line and add <agent_name> after it
			$SED -i "/<enrollment>/a      <agent_name>${AGENT_NAME}<\/agent_name>" $O_CONF 
			$SED -i "/<agent_name>/s/^/      /" $O_CONF
			app_error ${PIPESTATUS[0]} "  ossec_conf_update: agent_name add failed"
		fi
	else
		$SED -i "/<agent_name>/d" "$O_CONF"
		app_error ${PIPESTATUS[0]} "  ossec_conf_update: groups remove failed"
	fi

	if [ -n "$GROUP" ]; then
		if grep "<groups>" $O_CONF >/dev/null; then
			$SED -i "s/<groups>.*/<groups>${GROUP}<\/groups>/g" $O_CONF
			app_error ${PIPESTATUS[0]} "  ossec_conf_update: groups update failed"
		else
	        $SED -i "/<enrollment>/a      <groups>${GROUP}<\/groups>" "$O_CONF" 
			$SED -i "/<groups>/s/^/      /" "$O_CONF"
			app_error ${PIPESTATUS[0]} "  ossec_conf_update: groups add failed"
		fi
	else
		$SED -i "/<groups>/d" "$O_CONF"
		app_error ${PIPESTATUS[0]} "  ossec_conf_update: groups remove failed"
	fi

}

# Solaris 10 support
ossec_conf_update_legacy() {
    MSG="configuration failed for Hub server destination"

    # Write the ed commands to update <address> and <port> within <server>
    printf "/<server>/,/<\/server>/s|<address>.*</address>|<address>%s</address>|\n" "$SERVER_IP" > /tmp/ed_commands
    printf "/<server>/,/<\/server>/s|<port>.*</port>|<port>%s</port>|\n" "$PORT" >> /tmp/ed_commands
    printf "/<server>/,/<\/server>/s|<protocol>.*</protocol>|<protocol>%s</protocol>|\n" "$PROTOCOL" >> /tmp/ed_commands
    printf "w\nq\n" >> /tmp/ed_commands

    # Run ed with the generated commands
    ed -s "$O_CONF" < /tmp/ed_commands

    # Capture error status
    if [ $? -ne 0 ]; then
        echo "ERROR: $MSG"
        return 2
    fi

    # Cleanup
    rm -f /tmp/ed_commands
}




key_generate() {
	if [[ $NO_REGISTER -eq 1 ]]; then
		show_msg "Agent registration skipped" log
	else
  		password_auth
		# Add agent key
		if [ ! -s /var/ossec/etc/client.keys ]; then
			if [ "$USE_AWS" -eq 1 ]; then
				aws_register
			else
				if [ -z "$AGENT_NAME" ]; then
					if [ "$HTTP_ENROLL" = true ]; then
						#if [[ $PKG == "aix" || $PKG == "pkg" ]]; then
							request_agent_key ${AUTH_IP} 
						#else
						#	/var/ossec/bin/agent-auth -m ${AUTH_IP} -G ${GROUP} -I any -H |tee -a $LOG
						#fi
						app_error_authd ${PIPESTATUS[0]} "  key_generate: agent authorization failed"
					else
						/var/ossec/bin/agent-auth -m ${AUTH_IP} -G ${GROUP} -I any |tee -a $LOG
						app_error_authd ${PIPESTATUS[0]} "  key_generate: agent authorization failed"
					fi
				else
					if [ "$HTTP_ENROLL" = true ]; then
						#if [[ $PKG == "aix" || $PKG == "pkg" ]]; then
							request_agent_key ${AUTH_IP} 
						#else
						#	/var/ossec/bin/agent-auth -m ${AUTH_IP} -A ${AGENT_NAME} -G ${GROUP} -I any -H |tee -a $LOG
						#fi
						app_error_authd ${PIPESTATUS[0]} "  key_generate[named agent]: agent authorization failed"
					else
						/var/ossec/bin/agent-auth -m ${AUTH_IP} -A ${AGENT_NAME} -G ${GROUP} -I any |tee -a $LOG
						app_error_authd ${PIPESTATUS[0]} "  key_generate[named agent]: agent authorization failed"
					fi
				fi
			fi
			show_msg "Agent registered successfully" log
		else
			show_msg "Skipping, agent is already registered" log
		fi

		chown ossec:ossec /var/ossec/etc/client.keys
		chmod 640 /var/ossec/etc/client.keys
		chown root:ossec /var/ossec/etc
		chmod 770 /var/ossec/etc
	fi
}

yum_install () {
	# check for repo
	cat  << EOF > /etc/yum.repos.d/atomicorp-ossec.repo
[atomicorp-ossec]
baseurl = ${HTTP}://${SERVER_IP}/channels/awp-hub-repo/${DIR}/\$basearch
gpgcheck = 1
gpgkey = ${HTTP}://${SERVER_IP}/RPM-GPG-KEY.atomicorp.txt
name = Atomicorp Workload Protection
sslverify = 0
EOF

	# Test for selinux policy
	# Remove the SELinux policy, this handles the condition if /var/ossec was rm -rf'd
	if [ ! -d /var/ossec ]; then
		if command -v getenforce > /dev/null 2>&1 && command -v semodule > /dev/null 2>&1; then
		  if [ $(getenforce) != "Disabled" ]; then
		    if (semodule -l | grep ossec_agent > /dev/null); then
		      semodule -X 200 -r ossec_agent > /dev/null || :
		    fi
		  fi
		fi
	fi


	# Test for agent install
	if ! rpm -q ossec-hids-agent >/dev/null; then
		# If CLAMAV_EXCLUDE is set, exclude clamav packages
		if [[ $EXCLUDE_CLAMAV -eq 1 ]]; then
			yum --exclude=clamav* -y install libcurl ossec-hids-agent | tee -a $LOG
		else
			yum -y install libcurl ossec-hids-agent | tee -a $LOG	
		fi
    		app_error ${PIPESTATUS[0]} "yum failed during installation"
	else
    		show_msg "OSSEC Agent install detected"
	fi


	if [  ! -d /var/ossec/ ]; then
	    	echo
	    	show_msg "  ERROR: /var/ossec not detected. Client installation failure?"
	    	echo "  exiting..."
		echo
	    	exit 1
	fi

	ossec_conf_update
	key_generate

	if [[ $NO_REGISTER -eq 1 ]]; then
    		show_msg "Installation only: skipping start"
	else
		show_msg "Restarting OSSEC Agent"
		echo
		/var/ossec/bin/ossec-control restart
	fi
}

zypper_install () {

    # Add keys
    if [ !  -f /etc/pki/rpm-gpg/RPM-GPG-KEY.atomicorp.txt ]; then
        if [ ! -d /etc/pki/rpm-gpg ]; then
                mkdir -p /etc/pki/rpm-gpg/
        fi
	pushd /etc/pki/rpm-gpg/ >/dev/null
        	$GET ${HTTP}://${SERVER_IP}/RPM-GPG-KEY.atomicorp.txt >> $LOG 2>&1
	popd >/dev/null

        rpm --import /etc/pki/rpm-gpg/RPM-GPG-KEY.atomicorp.txt
    fi

    # Add repo
    zypper ar ${HTTP}://${SERVER_IP}/channels/awp-hub-repo/opensuse/15.1/x86_64/ atomic-ossec
    if [ $? -ne 0 ]; then
	    app_error ${PIPESTATUS[0]} "ERROR: zypper failed to add channel"
    fi



    # Install package
    /usr/bin/zypper --gpg-auto-import-keys -n install libcurl-devel ossec-hids-agent
    if [ $? -ne 0 ]; then
	    app_error ${PIPESTATUS[0]} "ERROR: osseec-hids-agent install failed"
    fi

    ossec_conf_update
    key_generate

    if [[ $NO_REGISTER -eq 1 ]]; then
   	show_msg "Installation only: skipping start"
    else
	# Start service
	service ossec-hids status >/dev/null 2>&1
	if [ $? -ne 0 ]; then
		/var/ossec/bin/ossec-control restart
	fi
    fi


}

# This function will enable unattended upgrades on an apt based system
# ex: enable_unattended_upgrades_for_repo "Atomicorp Agent Repository" "stable"
enable_unattended_upgrades_for_repo() {
    local origin="$1"
    local label="$2"
    local config_file="/etc/apt/apt.conf.d/99atomicorp-unattended-upgrades"

    # Ensure both origin and label are provided
    if [[ -z "$origin" || -z "$label" ]]; then
        echo "Usage: enable_unattended_upgrades_for_repo <origin> <label>"
        exit 1
    fi

    # Create or append to the custom configuration file
    echo "Unattended-Upgrade::Allowed-Origins {" > "$config_file" 
    echo "    \"${origin}:${label}\";" >> "$config_file" 
    echo "};" >> "$config_file" 

    echo "Custom unattended-upgrades configuration added for ${origin}:${label}"
}
apt_install () {


	if [ ! -f /usr/bin/gpg ]; then
	  	apt -y update | tee -a $LOG
		app_error ${PIPESTATUS[0]} "apt failed to update repodata"
	  	apt -y install gpg | tee -a $LOG
		app_error ${PIPESTATUS[0]} "apt failed to install gpg"
  	fi

  	if [ ! -f RPM-GPG-KEY.atomicorp.txt ]; then
    	$GET ${HTTP}://${SERVER_IP}/RPM-GPG-KEY.atomicorp.txt
		if [ $? -ne 0 ]; then
			$GET https://www.atomicorp.com/RPM-GPG-KEY.atomicorp.txt | tee -a $LOG
		fi
  	fi


    	if [ -d /etc/apt/trusted.gpg.d ]; then
		if [ ! -f /etc/apt/trusted.gpg.d/atomic.gpg ]; then
       			gpg --dearmor RPM-GPG-KEY.atomicorp.txt
       			app_error ${PIPESTATUS[0]} "could not dearmor GPG key, Exiting...."
			mv RPM-GPG-KEY.atomicorp.txt.gpg /etc/apt/trusted.gpg.d/atomic.gpg
			chmod 644 /etc/apt/trusted.gpg.d/atomic.gpg
       			app_error ${PIPESTATUS[0]} "could not install GPG key, Exiting...."
		fi
    	else
		# Legacy system
		cat RPM-GPG-KEY.atomicorp.txt | apt-key add -
		app_error ${PIPESTATUS[0]} "could not install GPG key (Legacy), Exiting...."
    	fi



  	if [ -d /etc/apt/sources.list.d/ ]; then
   		APT_SOURCES="/etc/apt/sources.list.d/atomicorp-ossec.list"
   		echo -n "Adding [atomicorp-ossec] to $APT_SOURCES: " | tee -a $LOG
   		echo "deb [arch=${ARCH} signed-by=/etc/apt/trusted.gpg.d/atomic.gpg] ${HTTP}://${SERVER_IP}/channels/awp-hub-repo/${DIST} ${DIR} main" > $APT_SOURCES
   		echo "OK" | tee -a $LOG
  	else
   		APT_SOURCES="/etc/apt/sources.list"
   		echo -n "Adding [atomicorp-ossec] to $APT_SOURCES: " | tee -a $LOG
   		echo "deb [arch=${ARCH} signed-by=/etc/apt/trusted.gpg.d/atomic.gpg ]] ${HTTP}://${SERVER_IP}/channels/awp-hub-repo/${DIST} ${DIR} main" >> /etc/apt/sources.list
   		echo "OK" | tee -a $LOG
  	fi


  	echo
  	show_msg "Updating system ... "
  	apt -y update | tee -a $LOG

  	if ! apt list --installed | grep -q ossec-hids-agent ; then
		DEBIAN_FRONTEND=noninteractive  apt install -o  Dpkg::Options::="--force-confmiss" -y libcurl4 libcurl4-openssl-dev ossec-hids-agent | tee -a $LOG
		app_error ${PIPESTATUS[0]} "Apt failed during installation."
  	else
   		show_msg "OSSEC Agent already installed."
  	fi

  	if [ ! -d /var/ossec/ ]; then
		echo
		show_msg "ERROR: /var/ossec not detected. Agent installation failure?"
  	fi

	ossec_conf_update
	key_generate

	if [[ $NO_REGISTER -eq 1 ]]; then
    	show_msg "Installation only: skipping start"
	else
		service ossec-hids restart >/dev/null 2>&1
		if [ $? -ne 0 ]; then
		    if [ -f /var/ossec/bin/ossec-control ]; then
				show_msg "Restarting OSSEC Agent"
				echo
				/var/ossec/bin/ossec-control restart > /dev/null 2>&1
			else
				service ossec start
		    fi
		fi
	fi

	# Enable unattended upgrades for the Atomicorp repository
	if [[ "$UNATTENDED_UPGRADES" -eq 1 ]]; then
		enable_unattended_upgrades_for_repo "Atomicorp Agent Repository" "stable"
	fi

}

function proxy_config() {
	if [[ $PROXY == "disable" ]]; then
		show_msg "  Proxy disabled" 
		unset http_proxy
		unset https_proxy
		if [[ "$PKG" = "deb" ]]; then
			show_msg "Bypassing proxy settings from apt in /etc/apt/apt.conf.d/atomcorp-proxy.conf" debug
 			echo "Acquire::http::Proxy {" > /etc/apt/apt.conf.d/atomcorp-proxy.conf
    		echo "   $SERVER_IP DIRECT;" >> /etc/apt/apt.conf.d/atomcorp-proxy.conf
    		echo "};" >> /etc/apt/apt.conf.d/atomcorp-proxy.conf
		elif [[ "$PKG" = "rpm" ]]; then
			show_msg "Removing proxy settings from /etc/yum.conf or /etc/dnf/dnf.conf" debug
			sed -i "/^proxy/d" "$(readlink -f /etc/yum.conf)"
		fi
	elif [[ $PROXY ]]; then
		# verify that $PROXY is a valid URL
		URL_PATTERN="^(http|https|ftp)://(([^:/?#]+)(:([^/?#]*))?@)?([^/?#]+)(/[^?#]*)?(\?([^#]*))?(#(.*))?$"
		if [[ $PROXY =~ $URL_PATTERN ]]; then
			show_msg "  Proxy URL: $PROXY " 
			if [[ "$PKG" = "deb" ]]; then
				echo "Acquire::http::Proxy {" > /etc/apt/apt.conf.d/atomcorp-proxy.conf
				echo "   $PROXY;" >> /etc/apt/apt.conf.d/atomcorp-proxy.conf
				echo "};" >> /etc/apt/apt.conf.d/atomcorp-proxy.conf
			elif [[ "$PKG" = "rpm" ]]; then
				if ! grep -q "proxy.*$PROXY" /etc/yum.conf; then
					echo "proxy=$PROXY" >> "$(readlink -f /etc/yum.conf)"
				fi
			fi
		else
			show_msg "Proxy URL is invalid" 
			show_msg "Exiting" log
			exit 1
		fi
	fi
}

solaris_install() {

    CPU_ARCH=$(uname -p)
    if [ "$CPU_ARCH" = "i386" ]; then
        CPU_ARCH="i386"
		PKG_ARCH="i86pc"
    else
        CPU_ARCH="sparc"
		PKG_ARCH="sparc"
    fi
    PKGUTIL_CONF="/etc/opt/csw/pkgutil.conf"
    MIRROR_URL="http://get.opencsw.org"


    # detect if pkgutil is installed
    echo
    echo "Dependencies can be installed from the public OpenCSW repository, or from the AEO hub server"
    echo "Enabling the Public OpenCSW repository is recommended."
    echo 

	if [[ $OPENCSW_REPO == "local" ]]; then
		INSTALL_PKGUTIL=n
	elif [[ $OPENCSW_REPO == "official" ]]; then
		INSTALL_PKGUTIL=y
	else
		# Interactive
    	read -p "  Would you like to install dependencies from http://get.opencsw.org? [y/n] (RECOMMENDED): " INSTALL_PKGUTIL
    	if [[ "$INSTALL_PKGUTIL" == "n" ]]; then
			read -p "  Press Enter to continue installation using the AEO hub server, or crtl-c to exit: " CONTINUE
		fi
	fi


	echo "instance=overwrite"> /var/tmp/admin.csw 
	echo "mail=" >> /var/tmp/admin.csw 
	echo "partial=quit">> /var/tmp/admin.csw 
	echo "runlevel=quit" >> /var/tmp/admin.csw
	echo "idepend=quit" >> /var/tmp/admin.csw
	echo "rdepend=quit" >> /var/tmp/admin.csw
	echo "space=quit" >> /var/tmp/admin.csw
	echo "setuid=nocheck">> /var/tmp/admin.csw 
	echo "conflict=quit" >> /var/tmp/admin.csw
	echo "action=nocheck" >> /var/tmp/admin.csw

    	if [[ "$INSTALL_PKGUTIL" == "n" ]]; then
		# Prompt to hit enter to continue
		HUB_IP=$SERVER_IP

		if [[ $HUB_IP == *http* ]]; then
      	 	echo "ERROR: Please provide only the IP address or hostname for the AEO hub server. Exiting..."
      	 	echo
       		exit 1
       	fi

	if [[ $HUB_IP ]]; then
         	# Install pkgutil from the provided URL
            	echo "    Installing pkgutil from the provided URL..."
		/usr/sfw/bin/wget http://$HUB_IP/channels/awp-hub-repo/solaris/opencsw/now
            if [[ $? -ne 0 ]]; then
				echo "ERROR: Failed to install pkgutil from the provided URL. Please check your network settings"
               	echo
               	exit 1
            fi

			echo all | pkgadd  -n -a /var/tmp/admin.csw -d ./now
            # Set pkgutiil to use the local mirror
            echo "mirror=http://$HUB_IP/channels/awp-hub-repo/solaris/opencsw/" > /etc/opt/csw/pkgutil.conf
                
        else   
            echo "ERROR: No Hub server provided. Exiting..."
            echo
            exit 1
        fi
    else 
	# Install pkgutil from the internet
        echo "    Installing pkgutil from the http://get.opencsw.org/now ..."

	/usr/sfw/bin/wget http://get.opencsw.org/now
	echo all | pkgadd  -n -a /var/tmp/admin.csw -d ./now
        # if this fails return an error that the internet cannot be reached
        if [[ $? -ne 0 ]]; then
            echo "ERROR: Failed to install pkgutil from [http://get.opencsw.org/now].  Please check your network settings."
            echo
            exit 1
        fi
    fi

    echo -n "Verifying connectivity: "

    if grep "^mirror=" "$PKGUTIL_CONF" >/dev/null; then
        MIRROR_URL=$(grep "^mirror=" "$PKGUTIL_CONF" | cut -d'=' -f2)
    fi

    if ! /usr/sfw/bin/wget --spider --timeout=10 --tries=2 "$MIRROR_URL" &>/dev/null; then
        echo "ERROR: Unable to communicate with the mirror ($MIRROR_URL). Please check your network settings. Exiting..."
        echo
        exit 1
    fi
    echo "Repo connectivity verified."



    # Insatll components
    DEPENDENCIES=("gawk" "gsed" "curl" "clamav")
    # if clamav is excluded, remove it from the dependencies
    if [[ $EXCLUDE_CLAMAV -eq 1 ]]; then
		DEPENDENCIES=("CSWlibgcc-s1")
    fi

    for DEP in "${DEPENDENCIES[@]}"; do
        echo "Installing $DEP..."
        /opt/csw/bin/pkgutil -y -i "$DEP"
        if [[ $? -ne 0 ]]; then
            read -p "Failed to install $DEP. Would you like to continue? [y/n]: " CONTINUE
            if [[ "$CONTINUE" != "y" ]]; then
                echo "Exiting installation process."
                echo
                exit 1
            fi
        fi
    done


    export PATH=/opt/csw/bin:$PATH
    SED=gsed
    AWK=gawk

	if pkginfo |grep ossec-hids-agent >/dev/null ; then
		echo "OSSEC Agent is already installed..."
	else
        local pkg_file="ossec-hids-agent-latest.sol11.art.${PKG_ARCH}.pkg"
        local url="${HTTP}://${SERVER_IP}/channels/awp-hub-repo/solaris/11/${PKG_ARCH}/${pkg_file}"

        show_msg "Attempting to download package from ${url}" log

		/usr/sfw/bin/wget "${url}"
        
        # Check if curl was successful
        if [[ $? -ne 0 ]]; then
            show_msg "ERROR: Failed to download package from ${url}. Check if the URL is correct or if the server is reachable." error
            exit 1
        fi

        # Confirm the file exists and is not empty
        if [[ ! -f "${pkg_file}" || ! -s "${pkg_file}" ]]; then
            show_msg "ERROR: The downloaded file (${pkg_file}) is either missing or empty." error
            exit 1
        fi
        show_msg "Successfully downloaded ${pkg_file}" log

        # Proceed with the installation
        show_msg "Installing the package ${pkg_file}" log
        echo all | pkgadd  -n -a /var/tmp/admin.csw -d ./${pkg_file}
        if [ ! -f /var/ossec/bin/ossec-agentd ]; then
            show_msg "ERROR: Failed to install package ${pkg_file} using pkgadd." error
            exit 1
        fi
        show_msg "Package ${pkg_file} installed successfully." log
	fi


	if pkginfo | grep awp-agent >/dev/null || [ "${EXCLUDE_CLAMAV}" = "1" ]; then
            echo "AWP Agent API is already installed, or installation skipped..."
	else
       	    local pkg_file="awp-agent-latest.sol11.art.all.pkg"
       	    local url="${HTTP}://${SERVER_IP}/channels/awp-hub-repo/solaris/11/${PKG_ARCH}/${pkg_file}"
       	    show_msg "Attempting to download package from ${url}" log
			/usr/sfw/bin/wget ${url}
        
            # Check if curl was successful
		if [[ $? -ne 0 ]]; then
		    show_msg "ERROR: Failed to download package from ${url}. Check if the URL is correct or if the server is reachable." error
		    exit 1
		fi

		# Confirm the file exists and is not empty
		if [[ ! -f "${pkg_file}" || ! -s "${pkg_file}" ]]; then
		    show_msg "ERROR: The downloaded file (${pkg_file}) is either missing or empty." error
		    echo "DEBUG: $url"
		    exit 1
		fi

		show_msg "Successfully downloaded ${pkg_file}" log

		show_msg "Installing the package ${pkg_file}" log
		echo all | pkgadd  -n -a /var/tmp/admin.csw -d ./${pkg_file}
		if [ $? -ne 0 ]; then
		    show_msg "ERROR: Failed to install package ${pkg_file} using pkgadd." error
		    exit 1
		fi
	fi

    # Update ossec.conf (IP, protocol, group/agent name)
    show_msg "Updating OSSEC configuration file (${O_CONF})" log
    ossec_conf_update_legacy
    if [[ $? -ne 0 ]]; then
        show_msg "ERROR: Failed to update ossec.conf with the new settings." error
        exit 1
    fi
    show_msg "OSSEC configuration updated successfully." log

    # Register agent with agent-auth
    show_msg "Registering agent with agent-auth" log
    key_generate

    # Start the OSSEC agent
    show_msg "Starting OSSEC Agent" log
    /var/ossec/bin/ossec-control restart
    if [[ $? -ne 0 ]]; then
        show_msg "ERROR: Failed to start the OSSEC agent." error
        exit 1
    fi
    show_msg "OSSEC Agent started successfully." log
}



###################
# Main
###################

if [[ $1 == -* ]]; then
    while [ $# -gt 0 ]; do
        case "$1" in
            --server)
                shift
                SERVER_IP=$1
                ;;
            --authserver)
                shift
                AUTH_IP=$1
                ;;
            --enable-unattended-upgrades)
                UNATTENDED_UPGRADES=1
                show_msg "called --enable-unattended-upgrades" log
                ;;
            --password)
                shift
                PASSWORD=$1
                ;;
            --protocol)
                shift
                PROTOCOL=$1
                ;;
            --port)
                shift
                PORT=$1
                ;;
            --proxy)
                shift
                PROXY=$1
                ;;
            --agent-name)
                shift
                AGENT_NAME=$1
                ;;
            --opencsw-repo)
                shift
                OPENCSW_REPO=$1
                ;;
            --exclude-clamav)
                EXCLUDE_CLAMAV=1
                show_msg "called --exclude-clamav" log
                ;;
            --group)
                shift
                GROUP=$1
                ;;
            --debug|-d)
                DEBUG=1
                ;;
            --https)
                HTTP="https"
                show_msg "called --use-https" log
                ;;
            --version|-v)
                echo "Atomic OSSEC Installer Version: ${VERSION}"
                exit
                ;;
            --help|-h)
                show_help
                exit 0
                ;;
            --https-enrollment)
                HTTP_ENROLL="true"
                show_msg "called --https-enrollment" log
                ;;
            *)
                echo "Unknown option: $1"
                ;;
        esac
        shift
    done
fi


# Legacy method
if [ ! ${SERVER_IP} ]; then
	echo "WARNING: Deprecated mode detected"
    SERVER_IP=$1
fi

if [ ! ${AUTH_IP} ]; then
    if [[ "$2" != *--* ]]; then
        AUTH_IP=$2
    fi
fi

if [ ! ${PROTOCOL} ]; then
        if [[ "$3" != *--* ]]; then
                PROTOCOL=$3
        fi
fi
if [ ! ${PORT} ]; then
        if [[ "$4" != *--* ]]; then
                PORT=$4
        fi
        if [ ! ${PORT} ]; then
                PORT=1514
        fi
fi

if [ ! ${GROUP} ]; then
        if [[ "$5" != *--* ]]; then
		GROUP=$5
	fi
fi


if [ ! ${SERVER_IP} ]; then
	show_help
	exit 1
fi

# Set defaults
if [ ! ${AUTH_IP} ]; then
	AUTH_IP=$SERVER_IP
fi

if [ ! ${PROTOCOL} ]; then
	PROTOCOL=udp
fi

if [ ! ${GROUP} ]; then
	GROUP=default
fi

if [ ! "$UID" ]; then
	UID=$(id -u)
fi

if [ "$UID" -ne "$ROOT_UID" ] ; then
	echo
	echo "  ERROR: You must be root to run this program."
	echo "  exiting..."
	echo
	exit 1
fi

echo
show_msg "Atomicorp OSSEC Agent installer ($VERSION)" 
show_msg "SERVER_IP=${SERVER_IP} AUTH_IP=${AUTH_IP} PROTOCOL=${PROTOCOL} GROUP=${GROUP}" log

if [[ $* == *--default-name* ]]; then
	#install dig
	externIP="$(dig +short myip.opendns.com @resolver1.opendns.com)"
	hostname="$(hostname)"
	AGENT_NAME=$externIP"-"$hostname
	show_msg "called --default-name" log
	show_msg "externIP=${externIP} hostname=${hostname} AGENT_NAME=${AGENT_NAME}" log
fi

if [[ $* == *--install-only ]]; then
	NO_REGISTER=1	
	show_msg "called --install-only" log
fi

if [[ $* == *--use-aws ]]; then
	USE_AWS=1
	show_msg "called --use-aws" log
fi



# Detect release/package type
PKG=rpm
SED=sed
AWK=awk
if [ -f /etc/redhat-release ]; then
        RELEASE=/etc/redhat-release
elif [ -f /etc/release ]; then
    	RELEASE=/etc/release
    	DIST="solaris"
    	PKG=pkg
	SED=gsed
	AWK=gawk
elif [[ $OSTYPE == "aix"* ]]; then
	PKG=aix
elif [ -f /etc/os-release ]; then
        RELEASE=/etc/os-release
elif [ -f /etc/openvz-release ]; then
        RELEASE=/etc/openvz-release
elif [ -f /etc/SuSE-release ]; then
        RELEASE=/etc/SuSE-release
elif [ -f /etc/os-release ]; then
        RELEASE=/etc/os-release
elif [ -f /etc/lsb-release ]; then
        RELEASE=/etc/lsb-release
elif [ -f /etc/debian_version ]; then
        RELEASE=/etc/debian_version
elif [ -f /etc/openvz-release ]; then
        RELEASE=/etc/openvz-release
elif [ -f /etc/virtuozzo-release ]; then
        RELEASE=/etc/virtuozzo-release
else
    echo "Error: unable to identify operating system"
    exit 1
fi

# Fallback for solaris
if [[ $(uname -s) == "SunOS" ]]; then
    	RELEASE=/etc/release
    	DIST="solaris"
    	PKG=pkg
	SED=gsed
	AWK=gawk
fi

if [[ $OSTYPE == "aix"* ]]; then
	PKG=aix
elif [[ $DIST == "solaris" ]]; then
	DIR=solaris/11
	PKG=pkg
	ARCH=$(arch)
elif egrep -q "(release 5)" $RELEASE ; then
	DIST="el5"
	DIR=centos/5
elif egrep -q "(release 6|release 2012)" $RELEASE ; then
	DIST="el6"
	DIR=centos/6
elif egrep -q "(release 7|release 2014)" $RELEASE ; then
	DIST="el7"
	DIR=centos/7
elif egrep -q "(release 8)" $RELEASE ; then
	DIST="el8"
	DIR=centos/8
elif egrep -q "(release 9)" $RELEASE ; then
    	DIST="el9"
    	DIR=centos/9
elif egrep -q "Red Hat Enterprise Linux.* 7" $RELEASE ; then
	DIST="el7"
	DIR=redhat/7
elif egrep -q "Red Hat Enterprise Linux.* 8" $RELEASE ; then
	DIST="el8"
	DIR=redhat/8
elif egrep -q "(Amazon Linux 2)" $RELEASE; then
	DIST="amazon"
	DIR=amazon/2
elif egrep -q "(Amazon Linux AMI|Amazon)" $RELEASE ; then
	DIST="el6"
	DIR=centos/6
elif egrep -q "wheezy" $RELEASE ; then
	DIST="debian"
	DIR="wheezy"
	PKG=deb
	ARCH=$(dpkg --print-architecture)
elif egrep -q "jessie" $RELEASE ; then
	DIST="debian"
	DIR="jessie"
	PKG=deb
	ARCH=$(dpkg --print-architecture)
elif egrep -q "stretch" $RELEASE ; then
	DIST="debian"
	DIR="stretch"
	PKG=deb
	ARCH=$(dpkg --print-architecture)
elif egrep -q "lucid" $RELEASE ; then
	DIST="debian"
	DIR="lucid"
	PKG=deb
	ARCH=$(dpkg --print-architecture)
elif egrep -q "precise" $RELEASE ; then
	DIST="debian"
	DIR="precise"
	PKG=deb
	ARCH=$(dpkg --print-architecture)
elif egrep -q "Raring Ringtail" $RELEASE ; then
	DIST="debian"
	DIR="raring"
	PKG=deb
	ARCH=$(dpkg --print-architecture)
elif egrep -q "Trusty Tahr" $RELEASE ; then
	DIST="ubuntu"
	DIR="trusty"
	PKG=deb
	ARCH=$(dpkg --print-architecture)
elif egrep -q "Xenial" $RELEASE ; then
	DIST="ubuntu"
	DIR="xenial"
	PKG=deb
	ARCH=$(dpkg --print-architecture)
elif egrep -q "Bionic" $RELEASE ; then
	DIST="ubuntu"
	DIR="bionic"
	PKG=deb
	ARCH=$(dpkg --print-architecture)
elif egrep -q "Focal Fossa" $RELEASE; then 
	DIST="ubuntu"
	DIR="focal"
	PKG=deb
	ARCH=$(dpkg --print-architecture)
elif egrep -q "Jammy Jellyfish" $RELEASE; then 
	DIST="ubuntu"
	DIR="jammy"
	PKG=deb
	ARCH=$(dpkg --print-architecture)
elif egrep -q "Noble Numbat" $RELEASE; then 
	DIST="ubuntu"
	DIR="noble"
	PKG=deb
	ARCH=$(dpkg --print-architecture)
elif egrep -q "buster" $RELEASE ; then
	DIST="debian"
	DIR="buster"
	PKG=deb
	ARCH=$(dpkg --print-architecture)
elif egrep -q "bullseye" $RELEASE ; then
	DIST="debian"
	DIR="bullseye"
	PKG=deb
	ARCH=$(dpkg --print-architecture)
elif egrep -q "bookworm" $RELEASE ; then
	DIST="debian"
	DIR="bookworm"
	PKG=deb
	ARCH=$(dpkg --print-architecture)
elif egrep -q "SUSE Linux Enterprise Server 12" $RELEASE; then
	DIST="suse15"
	DIR="opensuse/15.1"
	PKG=zypper
elif egrep -q "openSUSE Leap" $RELEASE; then
	DIST="suse15"
	DIR="opensuse/15.1"
	PKG=zypper
else
	echo "Error: Unable to determine distribution type. Please send the contents of $RELEASE to support@atomicorp.com"
	exit 1
fi

show_msg "RELEASE=$RELEASE DIST=$DIST DIR=$DIR PKG=$PKG ARCH=$ARCH" log

echo
proxy_config
detect_downloader

if [ "$PKG" == "rpm" ]; then
	# Only run hubcheck for RPM based systems
	hub_check
    	yum_install
	if [[ $NO_REGISTER -eq 1 ]]; then
  		show_msg "Installation only: skipping smoketest"
	else
		smoketest
	fi
elif [ "$PKG" == "deb" ]; then
    apt_install
    if [[ $NO_REGISTER -eq 1 ]]; then
        show_msg "Installation only: skipping smoketest"
    else
        smoketest
    fi
elif [ "$PKG" == "zypper" ]; then
    zypper_install
    if [[ $NO_REGISTER -eq 1 ]]; then
        show_msg "Installation only: skipping smoketest"
    else
        smoketest
    fi
elif [ "$PKG" == "pkg" ]; then
	solaris_install
elif [[ $PKG == "aix" ]]; then
  # Is RPM available
  if ! which rpm >/dev/null; then
    echo
    echo "Error: RPM not found. Exiting.... "
    exit 1
  fi

  # Is yum available
  if ! which yum >/dev/null; then
    echo
    echo "Error: Yum not found. Exiting.... "
    exit 1
  fi

  # Check for repo, and add
  if [ ! -d /opt/freeware/etc/yum.repos.d/ ]; then
    echo "Error: Yum repos.d could not be found. Exiting.... "
    exit 1
  fi

  REPOPATH=/opt/freeware/etc/yum.repos.d/


  if [ ! -f  ${REPOPATH}/atomicorp-ossec.repo ]; then
  	cat  << EOF > ${REPOPATH}/atomicorp-ossec.repo
[atomicorp-ossec]
baseurl = ${HTTP}://${SERVER_IP}/channels/awp-hub-repo/aix/7/ppc/
gpgcheck = 0
gpgkey = ${HTTP}://${SERVER_IP}/RPM-GPG-KEY.atomicorp.txt
name = Atomicorp OSSEC HIDS repo
sslverify = 0
EOF
  fi

  # Test for agent install
  if ! rpm -q ossec-hids-agent >/dev/null; then
	yum -y install libcurl ossec-hids-agent
    if [ $? -ne 0 ];then
      echo "ERROR: yum failed during installation"
      exit 1
    fi
  else
      echo
      echo "OSSEC Agent install detected"
      echo
  fi
  if [  ! -d /var/ossec/ ]; then
      echo
      echo "  ERROR: /var/ossec not detected. Client installation failure?"
      echo "  exiting..."
      echo
      exit 1
  fi


  password_auth
  # register with server
  if [ !  -f /var/ossec/etc/client.keys ]; then
	if [ "$HTTP_ENROLL" = true ]; then
		# if the platform is AIX or Solaris, use request_agent_key
		#if [[ $PKG == "aix" || $PKG == "pkg" ]]; then
			request_agent_key ${AUTH_IP} 
		#else
		#	/var/ossec/bin/agent-auth  -m ${AUTH_IP} -G ${GROUP} -H
		#fi
	else
		/var/ossec/bin/agent-auth  -m ${AUTH_IP} -G ${GROUP}
	fi

    if [ $? -ne 0 ]; then
      echo
      echo "  ERROR: agent-auth failed during registration"
      echo "  exiting..."
      exit 1
    fi
  fi

  # Update config
  if [ ! /opt/freeware/bin/sed ]; then
  	yum -y install sed
  fi
  O_CONF="/var/ossec/etc/ossec-agent.conf"
  export PATH=/opt/freeware/bin:$PATH
  ossec_conf_update

  # start service
  show_msg "Restarting OSSEC Agent"
  echo
  /var/ossec/bin/ossec-control restart

  # Add to startup
fi

show_msg "Installation Complete  . . . "
echo
